#include <stdlib.h>
#include <math.h>
#include "mex.h"

/* Dan Lofgreen and Stephen Eglen 2006
 * 
 * This software is distributed free under the conditions that
 * (1) it shall not be incorporated in software that is subsequently sold
 * (2) the authorship of the software shall be acknowledged in any 
 *     publication that uses results generated by the software
 * (3) this notice shall remain in place in each file.
 *
 * GPL applies: http://www.gnu.org/copyleft/gpl.html
 */

/* Compilation
 *
 * To compile this function in linux use:
 * mex dmin3d.c -lm
 *
 * To compile this mex function in Windows, use the following:
 * mex -v dmin3d.c
 *
*/


#define MAXREJECTS 999999
#define MAXDISTANCE 9999999	/* some large number used in nnd_n. */

/* This is the maximum number of cells that we can reject before
 * abandoning this simulation.  (This is the total through the simulation,
 * not the total per cell.)
 */

#define SMALL_DMIN_SD 1e-10	/* threshold for small s.d. value on dmin.  */

typedef double Sfloat;

/* Need some code for random number generation.*/

/* return random value uniformly distributed in [0,1] */
#define UNIF (rand()/(double)RAND_MAX)
#define RANDIN
#define RANDOUT
#ifndef M_PI
#define M_PI 3.14159265358979
#endif

/* Function definitons */
double norm_rand();		/* Normally distributed N(0,1). */

void dminlul3d(Sfloat *pwid, Sfloat *pht, Sfloat *pdepth, int *pnumcells,
	       Sfloat *pdmin, Sfloat *psd,
	       Sfloat *plower, Sfloat *pupper,
	       int *quiet,
	       Sfloat *xpts, Sfloat *ypts, Sfloat *zpts,
	       Sfloat *dmins, Sfloat *nrejects);

void nnd_n_3d(Sfloat *xs, Sfloat *ys, Sfloat *zs,
	      int n, Sfloat a, Sfloat b, Sfloat c,
	      int *idx, Sfloat *min);

void nnd_3d(Sfloat *xs, Sfloat *ys, Sfloat *zs, int n,
	    Sfloat a, Sfloat b, Sfloat c, int ignore,
	    int *idx, Sfloat *min);





void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
  int n, i, quiet,nn;
  double mean, sd;

  double *box;

  double lower, upper;
  double *pts;
  double *rejects;
  double *dmins, *info;
  double *xpts, *ypts, *zpts;
  double wid, ht, depth;
  
  if (nrhs != 3)
  mexErrMsgTxt("Three arguments please on RHS.");
    
  if (nlhs > 4)
    mexErrMsgTxt("At most 4 arguments please on LHS.");



  /*** Input parameters ***/ 
  n = (int)mxGetScalar(prhs[0]);

  box =  mxGetPr(prhs[1]);
  wid = box[0]; ht = box[1]; depth = box[2];
  /*printf("box size %f %f %f\n", wid, ht, depth);*/


  /* Dmin parameters */
  box =  mxGetPr(prhs[2]);
  mean = box[0]; sd = box[1]; lower = box[2]; upper = box[3];


  quiet = 0;
  plhs[1]=mxCreateDoubleMatrix(n,1,mxREAL);
  rejects = mxGetPr(plhs[1]);

  plhs[2]=mxCreateDoubleMatrix(n,1,mxREAL);
  dmins = mxGetPr(plhs[2]);

  plhs[3]=mxCreateDoubleMatrix(2,1,mxREAL);
  info = mxGetPr(plhs[3]);

  xpts = (double *) mxMalloc(n * sizeof(Sfloat));
  ypts = (double *) mxMalloc(n * sizeof(Sfloat));
  zpts = (double *) mxMalloc(n * sizeof(Sfloat));
  
  dminlul3d(&wid, &ht, &depth, &n,
	    &mean, &sd, &lower, &upper,
	    &quiet,
	    xpts, ypts, zpts,
	    dmins, rejects);

  /* Error handling: check if the simulation was successful. */
  if (xpts[0] <0) {
    /* Simulation was not successful. */
    plhs[0]=mxCreateDoubleMatrix(1,1,mxREAL);
    pts = mxGetPr(plhs[0]);
    *pts = -1;
    
  } else {
    /* Dmin was ok; copy across the results to the return array. */

    /* Create an mxArray for the output */
    plhs[0] = mxCreateDoubleMatrix(n, 3, mxREAL);
    pts = mxGetPr(plhs[0]);

    /* Return extra info re: the simulation. */
    info[0] = 0;		/* this will be number of rejects */

    /* info[1] will be packing density */
    info[1] = ((n * M_PI * 1/8 * mean * mean * mean)/
	       ( (3.0/4.0) * depth * wid * ht));
    
    nn= 2*n;
    for(i=0; i<n; i++) {
      pts[i]   = xpts[i];
      pts[i+n] = ypts[i];
      pts[i+nn]= zpts[i];
      info[0] += rejects[i];
    }
  }
  
  /* Free up what we don't need. */
  mxFree(xpts); mxFree(ypts); mxFree(zpts);

}


void dminlul3d(Sfloat *pwid, Sfloat *pht, Sfloat *pdepth, int *pnumcells,
	       Sfloat *pdmin, Sfloat *psd,
	       Sfloat *plower, Sfloat *pupper,
	       int *quiet,
	       Sfloat *xpts, Sfloat *ypts, Sfloat *zpts,
	       Sfloat *dmins, Sfloat *nrejects)

{
  /* 3-d version of Dmin rule.
   *
   * (P prefix on variable names indicates that variable is a pointer
   * to the value, not the value.)
   *
   * Create NUMCELLS cells distributed in an volume of size WID*HT*DEPTH.
   * This assumes one corner of the volume is the origin.
   *
   * The dmin value is chosen from a Normal distribution with mean
   * DMIN and s.d. of SD.  The dmin value is also bounded to lie
   * between LOWER and UPPER.  (LOWER is normally set to the size of a
   * cell body and UPPER can be ignored by setting it to a negative
   * number.)
   *
   * On exit, we return an array of cells stored in (XPTS, YPTS,
   * ZPTS).  If the simulation could not be performed (e.g. if DMIN
   * too high), the first value of XPTS is set to a negative value.
   * NJRECTS is a vector; NREJECTS[i] stores the number of cells that
   * were rejected when trying to position cell i.
   *
   * A new dmin value is created every time, regardless of whether
   * the trial cell was accepted or not.
   *
   * QUIET is a debugging option.
   */
  

  int num_rejects = 0, this_cell_rejects = 0;
  int looking, generate_dmin;
  int i, no_variance = 0;
  Sfloat x,y,z;
  Sfloat min; int idx;
  Sfloat this_dmin;
  Sfloat lower, upper;



  RANDIN;

  lower = *plower; upper = *pupper;

  if (*psd < SMALL_DMIN_SD) {
    /* Check once to see if the s.d. is too small to be useful; in this
     * case, all dmin values will be the same.
     */
    no_variance = 1;
  }
  for (i=0; i<*pnumcells; i++) {

    looking = 1; this_cell_rejects = 0;
    while (looking) {

      if (no_variance) {
	this_dmin = *pdmin;
	generate_dmin = 0;
      } else {
	generate_dmin = 1;
      }
      while (generate_dmin) {
	this_dmin = *pdmin + (*psd * norm_rand());
	if ( (this_dmin > lower) &&
	     ((upper <0) || (this_dmin < upper)))
	  generate_dmin = 0;
	/*else printf("dminlul: dmin %f outside range\n", this_dmin);*/
      }

      /*printf("this val %lf\n", this_dmin); */

      x = UNIF * (*pwid); y = UNIF * (*pht); z = UNIF * (*pdepth); 
      
      nnd_n_3d( xpts, ypts, zpts, i, x, y, z, &idx, &min);
      if ( (min < this_dmin) ) {
	/* || (min > upper)*/
	/* Cannot test if the min distance is too big... this won't
	 * work for early cells!  i.e. when putting the first cell in,
	 * the upper distance will be the value returned by nnd_n,
	 * ie.s. some huge number. */

	/* reject cell and try another. */
	num_rejects++;
	this_cell_rejects++;
	if (num_rejects > MAXREJECTS) {
	  /* If we cannot fit more cells in, return and indicate error
	   * by setting first x coordinate to negative value.
	   */
	  printf("dminlul error: num_rejects is too high\n");
	  xpts[0] = -1;
	  RANDOUT;
	  return;
	}
      }
      else {
	looking = 0;
      }
    }
    /* Accept this cell. */
    dmins[i] = this_dmin;
    xpts[i] = x; ypts[i] = y; zpts[i] = z;
    nrejects[i] = this_cell_rejects;
  }


  if (!*quiet) {
    printf("#rejects %d\tdminlul3d packing density %.3f\n", num_rejects,
	    ((*pnumcells * M_PI * 1/8 * *pdmin * *pdmin * *pdmin)/
	     ( (3.0/4.0) * *pdepth * *pwid * *pht)));
  }

  RANDOUT;
}

double norm_rand()
{

  double x1, x2, lnx1, z1;
  x1 = rand()/(double)RAND_MAX;
  x2 = rand()/(double)RAND_MAX;
  
  lnx1 = sqrt( -2*log(x1));

  z1 = lnx1 * cos(2*M_PI*x2);
  /* In principle, z2 could give us a 2nd number for free. */
  /*z2 = lnx1 * sin(2*M_PI*x2); */
  /*printf("%f %f\n", z1, z2); */

  return z1;
    
}

void nnd_n_3d(Sfloat *xs, Sfloat *ys, Sfloat *zs,
	      int n, Sfloat a, Sfloat b, Sfloat c,
	      int *idx, Sfloat *min)
{
  /* Return index (IDX) and distance (MIN) of cell closest to point (A,B,C).
   * The N datapoints are stored in XS[],  YS[], ZS[]/
   */
  if (n==0) {
    *idx = -1; *min = MAXDISTANCE;

    /* If n=0, no other cells have yet been positioned, so there is no
     * nearest neighbour. Hence, we just return some large distance to
     * indicate that fact. May be better to check that idx is less than zero,
     * since MAXDISTANCE may sometime be exceeded. */
  } else {
    nnd_3d(xs, ys, zs, n, a, b, c, -1, idx, min);
  }
}

void nnd_3d(Sfloat *xs, Sfloat *ys, Sfloat *zs, int n,
	    Sfloat a, Sfloat b, Sfloat c, int ignore,
	    int *idx, Sfloat *min)
{
  /* Return index (IDX) and distance (MIN) of cell closest to point (A,B,C).
   * The N datapoints are stored in XS[], YS[], ZS[] .
   * In comparison to nnd_n(), cell number IGNORE is not allowed to be
   * the closest cell.
   */
  int i;
  int index = -1; Sfloat min_dist = 0;

  int first = 1;		/* check for 1st distance comparison. */

  Sfloat x, y, z, dx, dy, dz, dist;
  
  for (i=0; i<n; i++) {
    if (i==ignore) {
      continue;
    }
    
    x = xs[i]; y = ys[i]; z = zs[i];
    dx = x - a; dy = y - b;  dz = z - c;
    dist = (dx*dx) + (dy*dy) + (dz*dz);
    if ( first || (dist < min_dist)) {
      first = 0;
      index = i;
      min_dist = dist;
    }
    
  }

  *min = sqrt(min_dist);
  *idx = index;
}
