#include "geom.h"
#include "func.h"

/*
	$Revision: 3.2 $	$Date: 91/01/03 18:38:56 $

	G function (nearest neighbour distribution) of 3D point pattern


	Let 	b = distance from point p[i] to boundary of box
	 	d = distance from p[i] to nearest p[j] 


	method = 1	naive ratio estimator (Ripley 1981)

			numerator(r)  = count(i: b >= r, d <= r)
			denominator(r)  = count(i: b >= r)

	method = 2	minus sampling estimator

			numerator(r) = count(i: b >= r, d <= r)
			denominator(r) = lambda * volume(x: b >= r)

			where lambda = (no of points)/volume(box)

	method = 3	Hanisch's G3

			numerator(r) = count(i: b >= d, d <= r)
			denominator(r) = count(i: b >= d)

	method = 4	Hanisch's G4

			numerator(r) = count(i: b >= d, d <= r)
			denominator(r) = fudge * volume(x: b >= r)

			fudge = numerator(R)/denominator(R)
			R = sup{r : denominator(r) > 0 }


# /////////////////////////////////////////////
# AUTHOR: Adrian Baddeley, CWI, Amsterdam, 1991.
# 
# This software is distributed free
# under the conditions that
# 	(1) it shall not be incorporated
# 	in software that is subsequently sold
# 	(2) the authorship of the software shall
# 	be acknowledged in any publication that 
# 	uses results generated by the software
# 	(3) this notice shall remain in place
# 	in each file.
# //////////////////////////////////////////////


*/

#define MIN(X,Y) (((X) > (Y)) ? (Y) : (X))

char *calloc();

double *
nndist(p, n, b)
		/* compute nearest neighbour distance for each p[i] */
	Point *p;
	int n;
	Box *b;
{
	register int i, j;
	register double dx, dy, dz, dist, nearest;
	Point *ip, *jp;
	double sqrt();
	double	*nnd;


	nnd = (double *) calloc((unsigned) n, sizeof(double));
	
	/* scan each point and find closest */
	for( i = 0; i < n; i++)
	{
		ip = p + i;
		for(j = 0; j < n; j++)
		if(j != i)
		{
			jp = p + j;
			dx = jp->x - ip->x;
			dy = jp->y - ip->y;
			dz = jp->z - ip->z;
			dist = sqrt(dx * dx + dy * dy + dz * dz);
			if(j == 0 || i == 0 && j == 1 
			|| dist < nearest)
				nearest = dist;
		}
		nnd[i] = nearest;
	}
	return(nnd);
}

double *
border(p, n, b)
		/* compute distances to border */
	Point *p;
	int n;
	Box *b;
{
	register int i;
	register double bord;
	register Point *ip;
	double *bored;

	bored = (double *) calloc((unsigned) n, sizeof(double));

	for( i = 0; i < n; i++)
	{
		ip = p + i;

		bord = MIN(ip->x - b->x0, b->x1 - ip->x);
		bord = MIN(bord, ip->y - b->y0);
		bord = MIN(bord, b->y1 - ip->y);
		bord = MIN(bord, ip->z - b->z0);
		bord = MIN(bord, b->z1 - ip->z);

		bored[i] = bord;
	}
	return(bored);
}

g3one(p, n, b, g)
Point *p;
int n;
Box *b;
Ftable *g;
{
	register int i, j, l, lbord, lnnd;
	double dt;
	double floor(), ceil();
	double	*bord, *nnd;

	bord = border(p, n, b);
	nnd  = nndist(p, n, b);
	
	/* initialise */
	for(l = 0; l < g->n; l++)
		(g->num)[l] = (g->denom)[l] = 0.0;

	/* spacing of argument in result vector g */
	dt = (g->t1 - g->t0)/(g->n - 1);

	for(i = 0; i < n; i++) { 

		lbord = floor( (bord[i] - g->t0) / dt );
		if(lbord >= g->n) 
			lbord = g->n - 1;
		for(l = 0; l <= lbord; l++)
			(g->denom)[l] += 1.0;

		lnnd = ceil( (nnd[i] - g->t0) / dt );
		if(lnnd < 0) lnnd = 0;
		for(l = lnnd; l <= lbord; l++)
			(g->num)[l] += 1.0;
	}

	/* compute ratio */
	for(l = 0; l < g->n; l++)
		(g->f)[l] = ((g->denom)[l] > 0)?
			(g->num)[l] / (g->denom)[l] : 1.0;
			   
}

g3three(p, n, b, g)
Point *p;
int n;
Box *b;
Ftable *g;
{
	register int i, j, l, lmin;
	double dt;
	int	denom;
	double ceil(), sqrt();
	double	*bord, *nnd;

	bord = border(p, n, b);
	nnd  = nndist(p, n, b);
	
	/* initialise */
	denom = 0;
	for(l = 0; l < g->n; l++)
		(g->num)[l]   = 0.0;

	/* spacing of argument in result vector g */
	dt = (g->t1 - g->t0)/(g->n - 1);

	for(i = 0; i < n; i++) { 

		if(nnd[i] <= bord[i]) {

			++denom;

			lmin = ceil( (nnd[i] - g->t0) / dt );
			if(lmin < 0) lmin = 0;
			for(l = lmin; l < g->n; l++)
					(g->num)[l] += 1.0;
		}
	}

	/* compute ratio */
	for(l = 0; l < g->n; l++)
	{
		(g->denom)[l] = denom;
		(g->f)[l] = (denom > 0)?
				(g->num)[l] / (double) denom
				: 1.0;
	}
			   
}
