function varargout = nearestneighbor3(p,varargin)
%NEARESTNEIGHBOR3 Calculates the nearest neighbor distances in 3D
%  NND = NEARESTNEIGHBOR3(P) Calculates the nearest neighbor distances for
%  each point in P.  P is an M x 3 matrix that contains the (x,y,z) 
%  coordinates of M points.  The function returns the nearest neighbor
%  distances in an array that is Mx1
%
%  [NND,MAXNNDIST,AVGNNDIST,NUMNN,NNINDICES,NNINDEX] = NEARESTNEIGHBOR3(P) 
%  Returns additional parameters: the maximum near neighbor distance, 
%  the average distance of the near neighbors, the number of near
%  neighbors, the indicies of each near neighbor, and the index of the
%  nearEST neighbor.
%
%  [...,T,DELAUNAYSEGLEN,DELAUNAYAREA,DELAUNAYTOTALAREA,DELAUNAYVOL,
%   SEGLENINDEX,FACETINDEX,NUMFACETS] =  NEARESTNEIGHBOR3(P)  Returns
%   additional parameters for the delaunay tessellation.
%
%     T                   Delaunay tessellation matrix T
%     DELAUNAYSEGLEN	  Delaunay segment lengths
%     DELAUNAYAREA	      Delaunay facet areas
%     DELAUNAYTOTALAREA	  Delaunay areas (for the tetrahedra)
%     DELAUNAYVOL	      Delaunay volumes 
%     SEGLENINDEX	      Indices of the points that make up the segments
%     FACETINDEX	      Indices of the points that make up the facetes
%     NUMFACETS	          Number of facets for each point 
%
%   See also: NNANALYSIS3, AUTOCORRELATION3, DELAUNAY3

% Perform a delaunay tessellation.  This does represent the number of 
% unique tetrahedrons
T = delaunayn(p);

% The first thing we should do is SORT the T matrix by columns so that the
% indices for each tetrahedron are in assending order.  This will guarentee
% that the indices for the facets and the segment lengths will also be in
% assending order.
T = sort(T,2);

% Then, we should sort T by rows so that when they are rendered they are
% rendered in some sort of meaningful order
T = sortrows(T);

% Call the C routine that calculates all the near neighbor and Delaunay
% statistics
[DelaunaySegLen,DelaunayArea,DelaunayTotalArea,DelaunayVolume,SegLenIndex,...
    FacetIndex,NumFacets,MinNNDist,MaxNNDist,AvgNNDist,NumNN,NNIndices,...
    NNIndex] = nnanalysis3(T,p);

% Find the unique set of facets
[FacetIndex,A] = unique(FacetIndex,'rows');
DelaunayArea = DelaunayArea(A);

% Find the unique set of segment lengths
[SegLenIndex,A] = unique(SegLenIndex,'rows');
DelaunaySegLen = DelaunaySegLen(A);

% Setup the output variables
varargout{1} = MinNNDist;

% Pass the rest of the near neighbor statistics if they were asked for
if(nargout>1)
    varargout{2} = MaxNNDist;
    varargout{3} = AvgNNDist;
    varargout{4} = NumNN;
    varargout{5} = NNIndices;
    varargout{6} = NNIndex;
end

% Pass the Delaunay statistics if they were asked for
if(nargout>5)
    varargout{7} = T;
    varargout{8} = DelaunaySegLen;
    varargout{9} = DelaunayArea;
    varargout{10} = DelaunayTotalArea;
    varargout{11} = DelaunayVolume;
    varargout{12} = SegLenIndex;
    varargout{13} = FacetIndex;
    varargout{14} = NumFacets;
end    

