function [tout,rout] = rosef(varargin)
%ROSEF   Filled Angle histogram plot.
%   ROSEF(THETA) plots the angle histogram as filled segments for the 
%   angles in THETA. The angles in the vector THETA must be specified in 
%   radians.
%
%   ROSEF(THETA,N) where N is a scalar, uses N equally spaced bins 
%   from 0 to 2*PI.  The default value for N is 20.
%
%   ROSEF(THETA,X) where X is a vector, draws the histogram using the
%   bins specified in X.
%
%   ROSEF(THETA,X,'NORM')  normalizes the count from the hist function 
%   to be in relative percentage.
%
%   ROSE(THETA,X,'NORM',F)  applies a weighting factor to each of the
%   counts from the hist function.  F must be the same length as X.  
%   Pass an empty matrix [] for 'NORM' if you do not wish to normalize 
%   the data.
%
%   ROSEF(AX,...) plots into AX instead of GCA.
%
%   H = ROSEF(...) returns a vector of line handles.
%
%   [T,R] = ROSEF(...) returns the vectors T and R such that 
%   POLAR(T,R) is the histogram.  No plot is drawn.
%
%   See also ROSE, HIST, POLAR, COMPASS.

%   Clay M. Thompson 7-9-91
%   Copyright 1984-2005 The MathWorks, Inc.
%   $Revision: Lofgreen $  $Date: 2006/06/21 $

[cax,args,nargs] = axescheck(varargin{:});
error(nargchk(1,4,nargs,'struct'));

theta = args{1};
if nargs > 1, 
  x = args{2}; 
end

if ischar(theta)
  error(id('NonNumericInput'),'Input arguments must be numeric.');
end
theta = rem(rem(theta,2*pi)+2*pi,2*pi); % Make sure 0 <= theta <= 2*pi
if nargs==1,
  x = (0:19)*pi/10+pi/20;

elseif nargs>1,
  if ischar(x)
    error(id('NonNumericInput'),'Input arguments must be numeric.');
  end
  if length(x)==1,
    x = (0:x-1)*2*pi/x + pi/x;
  else
    x = sort(rem(x(:)',2*pi));
  end

end
if ischar(x) || ischar(theta)
  error(id('NonNumericInput'),'Input arguments must be numeric.');
end

% Determine bin edges and get histogram
edges = sort(rem([(x(2:end)+x(1:end-1))/2 (x(end)+x(1)+2*pi)/2],2*pi));
edges = [edges edges(1)+2*pi];
nn = histc(rem(theta+2*pi-edges(1),2*pi),edges-edges(1));
nn(end-1) = nn(end-1)+nn(end);
nn(end) = [];

% Normalize the data here if asked.
if(nargs>2)
    if(nargs>3)
        F = args{4};
        if(length(F)~=length(nn)-1)
            error(id('ImproperDimension'),'Vector F must be the same length as X')
        end
    else
        F = ones(size(nn));
    end
    % Add the scaling factor
    nn = nn.*F;
    
    % Check t osee if the user passed the string 'NORM' and if so,
    % normalize the data
    norm = args{3};
    if(strcmp(norm,'NORM'))
        nn = nn./sum(nn)*100;
    end
end

% Form radius values for histogram triangle
if min(size(nn))==1, % Vector
  nn = nn(:); 
end
[m,n] = size(nn);
mm = 4*m;
r = zeros(mm,n);
r(2:4:mm,:) = nn;
r(3:4:mm,:) = nn;

% Form theta values for histogram triangle from triangle centers (xx)
zz = edges;

t = zeros(mm,1);
t(2:4:mm) = zz(1:m);
t(3:4:mm) = zz(2:m+1);

if nargout<2
  if ~isempty(cax)
    h = polarf(cax,t,r);
  else
    h = polarf(t,r);
  end
    
  if nargout==1, tout = h; end
  return
end

if min(size(nn))==1,
  tout = t'; rout = r';
else
  tout = t; rout = r;
end

function str=id(str)
str = ['MATLAB:rose:' str];
